@@ -13,6 +13,7 @@ import xmltodict
 
 from yojenkins.monitor import JobMonitor
 from yojenkins.utility import utility
+from yojenkins.utility.utility import fail_out, failures_out
 from yojenkins.yo_jenkins.jenkins_item_classes import JenkinsItemClasses
 from yojenkins.yo_jenkins.jenkins_item_config import JenkinsItemConfig
 
@@ -43,7 +44,7 @@ class Job():
         self.search_results = []
         self.search_items_count = 0
 
-    def __recursive_search(self, search_pattern: str, search_list: list, level: int, fullname: bool = True) -> None:
+    def _recursive_search(self, search_pattern: str, search_list: list, level: int, fullname: bool = True) -> None:
         """Recursive search method for jobs
 
         Details: Matched pattern findings are storred in the object: `self.search_results`
@@ -90,7 +91,7 @@ class Job():
                 continue
 
             # Keep searching all sub-items for this item. Call itself for some recursion fun
-            self.__recursive_search(search_pattern, list_item['jobs'], level, fullname)
+            self._recursive_search(search_pattern, list_item['jobs'], level, fullname)
 
     def search(self,
                search_pattern: str,
@@ -129,22 +130,21 @@ class Job():
                 items = self.jenkins_sdk.get_all_jobs(folder_depth=folder_depth)
             except jenkins.JenkinsException as error:
                 error_no_html = error.args[0].split("\n")[0]
-                logger.debug(f'Error while getting all items. Exception: {error_no_html}')
-
-                # TODO: Catch authentication error "[401]: Unauthorized"
-                return [], []
+                fail_out(f'Error while getting all items. Exception: {error_no_html}')
+            except Exception as error:
+                fail_out(error)
 
         # Search for any matching folders ("jobs")
         self.search_results = []
         self.search_items_count = 0
-        self.__recursive_search(search_pattern=search_pattern, search_list=items, level=0, fullname=fullname)
+        self._recursive_search(search_pattern=search_pattern, search_list=items, level=0, fullname=fullname)
 
         # Remove duplicates from list
         logger.debug('Removing duplicates if needed ...')
         self.search_results = [i for n, i in enumerate(self.search_results) if i not in self.search_results[n + 1:]]
 
         # Getting only the URLs of the stages
-        job_search_results_list = [r['url'] for r in self.search_results]
+        job_search_results_list = [result['url'] for result in self.search_results]
 
         # Output search stats
         logger.debug(
@@ -162,8 +162,7 @@ class Job():
             TODO
         """
         if not job_name and not job_url:
-            logger.debug('Failed to get job information. No job name or job url received')
-            return {}
+            fail_out('No job name or job URL provided')
 
         if job_name and not job_url:
             job_url = utility.name_to_url(self.rest.get_server_url(), job_name)
@@ -171,13 +170,11 @@ class Job():
         logger.debug(f'Job url passed: {job_url}')
         job_info, _, success = self.rest.request(f'{job_url.strip("/")}/api/json', 'get', is_endpoint=False)
         if not success:
-            logger.debug(f'Failed to find job info: {job_url}')
-            return {}
+            fail_out(f'Failed to find job info: {job_url}')
 
         # Check if found item type/class
         if job_info['_class'] not in JenkinsItemClasses.JOB.value['class_type']:
-            logger.debug(f'Failed to match type/class. This item is "{job_info["_class"]}"')
-            return {}
+            fail_out(f'Job found, but failed to match type/class. The found item is "{job_info["_class"]}"')
 
         if 'url' in job_info:
             job_info['fullName'] = utility.url_to_name(job_info['url'])
@@ -204,8 +201,6 @@ class Job():
         """
         # Get the job information
         job_info = self.info(job_name=job_name, job_url=job_url)
-        if not job_info:
-            return [], []
 
         # Get all the past builds
         build_list, build_url_list = utility.item_subitem_list(
@@ -227,10 +222,9 @@ class Job():
         """
         # Get the job information
         job_info = self.info(job_name=job_name, job_url=job_url)
-        if not job_info:
-            return None
 
-        # TODO: Check if nextBuildNumber is even part of the info
+        if not job_info.get('nextBuildNumber'):
+            fail_out('Failed to get next build number from job. "builds" key missing in job information')
 
         return job_info['nextBuildNumber']
 
@@ -249,15 +243,11 @@ class Job():
         if not job_info:
             # If the job info is not passed, request it from server
             job_info = self.info(job_name=job_name, job_url=job_url)
-            if not job_info:
-                return 0
 
         if 'lastBuild' not in job_info:
-            logger.debug('Failed to find "lastBuild" key in job info')
-            return 0
+            fail_out('Failed to get last build number from job. "lastBuild" key missing in job information')
         if 'number' not in job_info['lastBuild']:
-            logger.debug('Failed to find "number" key in "lastBuild" section of job info')
-            return 0
+            fail_out('Failed to get last build number from job. "lastBuild.number" key missing in job information')
 
         return job_info['lastBuild']['number']
 
@@ -271,8 +261,7 @@ class Job():
             TODO
         """
         if not job_name and not job_url:
-            logger.debug('Failed to set job next build number. No job name or job url received')
-            return None
+            fail_out('No job name or job URL provided')
         if job_url and not job_name:
             job_name = utility.url_to_name(url=job_url)
         # Format name
@@ -282,12 +271,11 @@ class Job():
 
         try:
             # TODO: Use requests instead of jenkins-python
-            response = self.jenkins_sdk.set_next_build_number(job_name, build_number)
+            self.jenkins_sdk.set_next_build_number(job_name, build_number)
         except jenkins.JenkinsException as error:
             error_no_html = error.args[0].split("\n")[0]
-            logger.debug(
+            fail_out(
                 f'Failed to set next build number for job "{job_name}" to {build_number}. Exception: {error_no_html}')
-            return None
 
         return build_number
 
@@ -307,14 +295,9 @@ class Job():
         if not job_info:
             # Getting job information
             job_info = self.info(job_name=job_name, job_url=job_url)
-            if not job_info:
-                logger.debug(f'Failed to find job "{job_name if job_name else job_url}"')
-                return None
 
         if 'builds' not in job_info:
-            logger.debug('Failed to get build list from job. "builds" key missing in job information')
-            logger.debug(f'Job info: {job_info}')
-            return None
+            fail_out('Failed to get build list from job. "builds" key missing in job information')
 
         # Iterate through all listed builds
         for build in job_info['builds']:
@@ -335,8 +318,7 @@ class Job():
         # NOTE: The jenkins-python module build_job() does not work. Using requests instead
 
         if not job_name and not job_url:
-            logger.debug('Failed to get trigger job build. No job name or job url received')
-            return 0
+            fail_out('No job name or job URL provided')
 
         logger.debug(f'Job reference passed: {job_name if job_name else job_url}')
 
@@ -374,7 +356,7 @@ class Job():
             logger.debug(f'Build queue URL: {queue_location}')
             logger.debug(f'Build queue ID: {build_queue_number}')
         else:
-            return 0
+            fail_out('Failed to trigger build. Failed to send request')
 
         return build_queue_number
 
@@ -407,8 +389,7 @@ class Job():
         elif build_queue_url:
             endpoint = f'{build_queue_url}api/json'
         else:
-            logger.error('No build queue number or build queue url passed')
-            return {}
+            fail_out('No build queue number or build queue url passed')
         queue_info = self.rest.request(endpoint, 'get', is_endpoint=True)[0]
 
         # Adding additional parameters
@@ -434,8 +415,7 @@ class Job():
             TODO
         """
         if not job_name and not job_url:
-            logger.debug('Failed to get job information. No job name or job url received')
-            return {}
+            fail_out('No job name or job URL provided')
 
         # Requesting all queue and searching queue (NOTE: Could use Server object)
         queue_all = self.rest.request('queue/api/json', 'get')[0]
@@ -470,23 +450,21 @@ class Job():
         logger.debug(f'Aborting build queue "{build_queue_number}" ...')
 
         if not build_queue_number:
-            logger.error('No build queue number passed')
-            return False
+            fail_out('No build queue number passed')
 
         # Make the request URL
         endpoint = f'queue/cancelItem?id={build_queue_number}'
         return_content = self.rest.request(endpoint, 'post', is_endpoint=True)[0]
 
         if not return_content:
-            logger.error(
-                'Failed to abort build queue. Specified build queue number may be wrong or build may have already started'
-            )
-            logger.error('The following jobs are currently in queue:')
+            messages = [
+                'Failed to abort build queue. Specified build queue number may be wrong or build may have already started',
+                'The following jobs are currently in queue:'
+            ]
             queue_list = self.in_queue_check()
             for i, queue_item in enumerate(queue_list):
-                logger.error(f'  {i+1}. Queue ID: {queue_item["id"]} - Job URL: {queue_item["task"]["url"]}')
-
-            return False
+                messages.append(f'  {i+1}. Queue ID: {queue_item["id"]} - Job URL: {queue_item["task"]["url"]}')
+            failures_out(messages)
 
         return True
 
@@ -500,8 +478,7 @@ class Job():
             TODO
         """
         if not job_name and not job_url:
-            logger.debug('Failed to get job information. No job name or job url received')
-            return False
+            fail_out('No job name or job URL provided')
 
         if job_url:
             job_url = job_url.strip('/')
@@ -509,9 +486,11 @@ class Job():
             job_url = utility.name_to_url(self.rest.get_server_url(), job_name)
 
         logger.debug(f'Opening in web browser: "{job_url}" ...')
-        success = utility.browser_open(url=job_url)
-        logger.debug('Successfully opened in web browser' if success else 'Failed to open in web browser')
-        return success
+        if not utility.browser_open(url=job_url):
+            fail_out('Failed to open job in web browser')
+        logger.debug('Successfully oped job in web browser')
+
+        return True
 
     def config(self,
                filepath: str = '',
@@ -519,7 +498,7 @@ class Job():
                job_url: str = '',
                opt_json: bool = False,
                opt_yaml: bool = False,
-               opt_toml: bool = False) -> Tuple[str, bool]:
+               opt_toml: bool = False) -> str:
         """Get the folder XML configuration (config.xml)
 
         Args:
@@ -529,11 +508,9 @@ class Job():
 
         Returns:
             Folder config.xml contents
-            True if configuration written to file, else False
         """
         if not job_name and not job_url:
-            logger.debug('Failed to get job information. No job name or job url received')
-            return '', False
+            fail_out('No job name or job URL provided')
 
         if job_url:
             job_url = job_url.strip('/')
@@ -545,14 +522,16 @@ class Job():
                                                        'get',
                                                        json_content=False,
                                                        is_endpoint=False)
-        logger.debug('Successfully fetched XML configurations' if success else 'Failed to fetch XML configurations')
+        if not success:
+            fail_out('Failed to get job configuration')
+        logger.debug('Successfully fetched XML configurations')
 
         if filepath:
             write_success = utility.write_xml_to_file(return_content, filepath, opt_json, opt_yaml, opt_toml)
             if not write_success:
-                return "", False
+                fail_out('Failed to write configuration file')
 
-        return return_content, True
+        return return_content
 
     def disable(self, job_name: str = '', job_url: str = '') -> bool:
         """TODO Docstring
@@ -564,8 +543,7 @@ class Job():
             TODO
         """
         if not job_name and not job_url:
-            logger.debug('Failed to get job information. No job name or job url received')
-            return False
+            fail_out('No job name or job URL provided')
 
         if job_url:
             job_url = job_url.strip('/')
@@ -574,7 +552,10 @@ class Job():
 
         logger.debug(f'Disabling job: "{job_url}" ...')
         success = self.rest.request(f'{job_url.strip("/")}/disable', 'post', is_endpoint=False)[2]
-        logger.debug('Successfully disabled job' if success else 'Failed to disable job')
+        if not success:
+            fail_out('Failed to disable job')
+        logger.debug('Successfully disabled job')
+
         return success
 
     def enable(self, job_name: str = '', job_url: str = '') -> bool:
@@ -587,8 +568,7 @@ class Job():
             TODO
         """
         if not job_name and not job_url:
-            logger.debug('Failed to get job information. No job name or job url received')
-            return False
+            fail_out('No job name or job URL provided')
 
         if job_url:
             job_url = job_url.strip('/')
@@ -597,7 +577,10 @@ class Job():
 
         logger.debug(f'Enabling job: "{job_url}" ...')
         success = self.rest.request(f'{job_url.strip("/")}/enable', 'post', is_endpoint=False)[2]
-        logger.debug('Successfully enabled job' if success else 'Failed to enable job')
+        if not success:
+            fail_out('Failed to enable job')
+        logger.debug('Successfully enabled job')
+
         return success
 
     def rename(self, new_name: str, job_name: str = '', job_url: str = '') -> bool:
@@ -610,8 +593,7 @@ class Job():
             TODO
         """
         if not job_name and not job_url:
-            logger.debug('Failed to get job information. No job name or job url received')
-            return False
+            fail_out('No job name or job URL provided')
 
         if job_url:
             job_url = job_url.strip('/')
@@ -619,14 +601,16 @@ class Job():
             job_url = utility.name_to_url(self.rest.get_server_url(), job_name)
 
         if not new_name:
-            logger.debug('New job name is a blank')
-            return False
+            fail_out('The new job name is a blank')
         if utility.has_special_char(new_name):
-            return False
+            fail_out('The new job name contains special characters')
 
         logger.debug(f'Renaming job: "{job_url}" ...')
         success = self.rest.request(f'{job_url.strip("/")}/doRename?newName={new_name}', 'post', is_endpoint=False)[2]
-        logger.debug('Successfully renamed job' if success else 'Failed to rename job')
+        if not success:
+            fail_out('Failed to rename job')
+        logger.debug('Successfully renamed job')
+
         return success
 
     def delete(self, job_name: str = '', job_url: str = '') -> bool:
@@ -639,8 +623,7 @@ class Job():
             TODO
         """
         if not job_name and not job_url:
-            logger.debug('Failed to get job information. No job name or job url received')
-            return False
+            fail_out('No job name or job URL provided')
 
         if job_url:
             job_url = job_url.strip('/')
@@ -649,7 +632,10 @@ class Job():
 
         logger.debug(f'Deleting job: "{job_url}" ...')
         success = self.rest.request(f'{job_url.strip("/")}/doDelete', 'post', is_endpoint=False)[2]
-        logger.debug('Successfully deleted job' if success else 'Failed to delete job')
+        if not success:
+            fail_out('Failed to delete job')
+        logger.debug('Successfully deleted job')
+
         return success
 
     def wipe_workspace(self, job_name: str = '', job_url: str = '') -> bool:
@@ -662,8 +648,7 @@ class Job():
             TODO
         """
         if not job_name and not job_url:
-            logger.debug('Failed to get job information. No job name or job url received')
-            return False
+            fail_out('No job name or job URL provided')
 
         if job_url:
             job_url = job_url.strip('/')
@@ -672,7 +657,10 @@ class Job():
 
         logger.debug(f'Wiping workspace for job: "{job_url}" ...')
         success = self.rest.request(f'{job_url.strip("/")}/doWipeOutWorkspace', 'post', is_endpoint=False)[2]
-        logger.debug('Successfully wiped job workspace' if success else 'Failed to wipe job workspace')
+        if not success:
+            fail_out('Failed to wipe workspace')
+        logger.debug('Successfully wiped workspace')
+
         return success
 
     def monitor(self, job_name: str = '', job_url: str = '', sound: bool = False) -> bool:
@@ -685,20 +673,22 @@ class Job():
             TODO
         """
         if not job_name and not job_url:
-            logger.debug('Failed to get job information. No job name or job url received')
-            return False
+            fail_out('No job name or job URL provided')
 
         if job_url:
             job_url = job_url.strip('/')
         else:
             job_url = utility.name_to_url(self.rest.get_server_url(), job_name)
 
+        if not self.rest.request(f'{job_url.strip("/")}/api/json', 'head', is_endpoint=False)[2]:
+            fail_out(f'Failed to find job. The job may not exist: {job_url}')
+
         logger.debug(f'Starting monitor for: "{job_url}" ...')
         success = self.JM.monitor_start(job_url=job_url, sound=sound)
-        if success:
-            logger.debug('Successfully opened monitor')
-        else:
-            logger.debug('Failed to open monitor for build')
+        if not success:
+            fail_out('Failed to start job monitor')
+        logger.debug('Successfully started job monitor')
+
         return success
 
     def create(self,
@@ -716,8 +706,7 @@ class Job():
             TODO
         """
         if not folder_name and not folder_url:
-            logger.debug('Failed to get folder information. No folder name or folder url received')
-            return False
+            fail_out('No folder name or folder url received')
 
         if folder_url:
             folder_url = folder_url.strip('/')
@@ -725,14 +714,13 @@ class Job():
             folder_url = utility.name_to_url(self.rest.get_server_url(), folder_name)
 
         if not name:
-            logger.debug('Item name is a blank')
-            return False
+            fail_out('Provided item name is a blank')
         if utility.has_special_char(name):
-            return False
+            fail_out('Provided item name contains special characters')
 
         # Check if job already exists
         if utility.item_exists_in_folder(name, folder_url, "job", self.rest):
-            return False
+            fail_out(f'Job "{name}" already exists in folder "{folder_url}"')
 
         if config_file:
             # Use job config from file
@@ -741,16 +729,14 @@ class Job():
                 open_file = open(config_file, 'rb')
                 job_config = open_file.read()
             except (OSError, IOError, PermissionError) as error:
-                logger.debug(f'Failed to open and read file. Exception: {error}')
-                return False
+                fail_out(f'Failed to open and read file. Exception: {error}')
 
             if config_is_json:
                 logger.debug('Converting the specified JSON file to XML format ...')
                 try:
                     job_config = xmltodict.unparse(json.loads(job_config))
                 except ValueError as error:
-                    logger.debug(f'Failed to convert the specified JSON file to XML format. Exception: {error}')
-                    return False
+                    fail_out(f'Failed to convert the specified JSON file to XML format. Exception: {error}')
         else:
             # Use blank job config template
             job_config = JenkinsItemConfig.JOB.value['blank']
